<?php

namespace DMG\Tiles\Blocks;

/**
 * Class Tiles
 *
 * Register the assets for the parent tiles block
 *
 * @package DMG\Tiles\Blocks
 */
class Tiles {

	/**
	 * Registered blocks
	 *
	 * @var array
	 */
	public $blocks = array();

	/**
	 * Init method
	 */
	public function __construct() {
		add_action( 'init', array( $this, 'register_block' ) );
		add_action( 'init', array( $this, 'register_child_blocks' ) );
		add_action( 'enqueue_block_editor_assets', array( $this, 'enqueue_block_editor_assets' ) );
		add_filter( 'upload_mimes', array( $this, 'upload_types' ), 99 );
	}

	/**
	 * Register the block
	 */
	public function register_block() {
		register_block_type(
			__DIR__,
			array(
				'render_callback' => array( $this, 'render' ),
			)
		);
	}

	/**
	 * Register the child blocks
	 */
	public function register_child_blocks() {

		$blocks = array();

		// Register core blocks
		$blocks['dmg/article-tile'] = 'DMG\Tiles\Blocks\Inner_Blocks\Article';

		/**
		 * Filter the registered tiles
		 *
		 * @param array $blocks The current registered tiles
		 */
		$this->blocks = apply_filters( 'dmg_tiles', $blocks );

		// Instantiate the classes after filtering
		foreach ( $this->blocks as $key => $class ) {
			new $class;
		}
	}

	/**
	 * Enqueue block scripts
	 */
	public function enqueue_block_editor_assets() {

		// Enqueue on allowed post types
		if ( ! in_array( get_post_type(), self::get_allowed_post_types(), true ) ) {
			return;
		}

		// Get asset data
		$asset_data = include( DMG_TILES_ROOT . '/build/dmg-tiles.asset.php' );

		wp_enqueue_script(
			'dmg-tiles',
			DMG_TILES_URL . '/build/dmg-tiles.js',
			$asset_data['dependencies'],
			$asset_data['version'],
			true
		);

		wp_enqueue_style(
			'dmg-tiles',
			DMG_TILES_URL. '/build/dmg-tiles.css',
			array(),
			$asset_data['version']
		);

		/**
		 * Filter the editor config
		 *
		 * @param array $config The current editor config
		 */
		$config = apply_filters(
			'dmg_tiles_editor_config',
			array(
				'allowedBlocks' => array_keys( $this->blocks ),
			)
		);

		/**
		 * Print some data on the page
		 */
		wp_localize_script(
			'dmg-tiles',
			'DMGTiles',
			$config
		);
	}

	/**
	 * Get allowed post types
	 *
	 * @return array The array of allowed post types
	 */
	public static function get_allowed_post_types() {

		/**
		 * Filter for post types to show tiles block on
		 * 
		 * @param array $post_types The allowed post types
		 */
		$post_types = apply_filters(
			'dmg_tiles_post_types',
			array(
				'page'
			)
		);

		return $post_types;
	}

	/**
	 * Allow SVG uploads
	 *
	 * @param array $mimes Currently permitted MIME types
	 * @return array The filtered allowable MIME types
	 */
	public function upload_types( $mimes ) {
		$mimes['svg'] = 'image/svg+xml';

		return $mimes;
	}

	/**
	 * Render function
	 *
	 * @param array  $attributes The blocks attributes
	 * @param string $content    Data returned from InnerBlocks.Content
	 * @param array  $block      Block information such as context.
	 * @return string The rendered block markup.
	 */
	public function render( $attributes, $content, $block ) {
		ob_start();
		?>
		<div class="dmg-tiles dmg-tiles--<?php echo esc_attr( $attributes['layout'] ); ?>">
			<?php echo $content; ?>
		</div>
		<?php
		return ob_get_clean();
	}

}

new Tiles();
