/**
 * WordPress dependencies
 */
import { TextControl, Spinner } from '@wordpress/components';
import { useState, RawHTML } from '@wordpress/element';
import { useEntityRecords } from '@wordpress/core-data';

/**
 * External depedencies
 */
import { debounce } from 'lodash';

// Declare the number of search results per page
const RESULTS_PER_PAGE = 10;

/**
 * Article selector component
 */
const ArticleSelector = ( { postType, params, selected, onChange } ) => {

	// Create some states
	const [ searchTerm, setSearchTerm ] = useState( '' );
	const [ page, setPage ] = useState( 1 );

	let isResolving = false;

	let articles = [];

	// Check if this is a regular, alphanumeric search term
	if ( searchTerm === '' || isNaN( searchTerm ) ) {

		// Build array of query parameters
		const additionalParams = {
			page
		};

		if ( searchTerm !== '' ) {
			additionalParams.search = searchTerm;

			// Restrict to last 3 months
			const date = new Date();
			date.setMonth(date.getMonth() - 3);
			additionalParams.after = `${date.toLocaleDateString('en-CA')}T00:00:00`
		}

		const queryParams = { ...params, ...additionalParams };

		// Fetch posts
		const fetchPosts = useEntityRecords( 'postType', 'post', queryParams );
		if (fetchPosts?.isResolving === true) {
			isResolving = true;
		} else {
			if (fetchPosts?.hasResolved && fetchPosts.records) {
				articles = fetchPosts.records;
			}
		}
	} else {

		// Found a number, try to query by article ID
		const fetchPost = useEntityRecords( 'postType', 'post', {include: searchTerm} );
		if (fetchPost?.isResolving === true) {
			isResolving = true;
		} else {
			if (fetchPost?.hasResolved && fetchPost.records) {
				articles = fetchPost.records;
			}
		}
	}

	/**
	 * Generate list of articles to display
	 */	
	const articleDisplay = ( article ) => {

		let className = 'dmg-tile-article-selector__article ';
		className += ( selected === article.id ) ? 'dmg-tile-article-selector__article-selected' : '';

		return (
			<li className={ className } onClick={ () => onChange( article ) }>
				<div className="dmg-tile-article-selector__article-title">
					<RawHTML>{ article.title.rendered }</RawHTML>
				</div>
				<date>
					{ new Date( Date.parse( article.date ) ).toDateString() }
				</date>
			</li>
		);
	};

	/**
	 * Debounced search function
	 */
	const search = debounce( (searchInput ) => {
		setSearchTerm( searchInput );
		setPage( 1 );
	}, 500 );

	/**
	 * Reset pagination on filter toggle
	 */
	const onFilterChange = (value) => {
		setNewspaperFilter( value );
		setPage( 1 );
	}

	// Determine if we need to hide any navigation
	const navPreviousClassname = ( page > 1 ) ? '' : 'hidden';
	const navNextClassname     = ( articles?.length === RESULTS_PER_PAGE ) ? '' : 'hidden';

	return (
		<div className="dmg-tile-article-selector">
			<div className="dmg-tile-article-selector__search">
				<TextControl
					onChange={ search }
					placeholder="Enter a search term or article ID"
				/>
			</div>
			{ isResolving &&
				<>
					<Spinner />
					<span>
						<em>Loading...</em>
					</span>
				</>
			}
			{ !isResolving && articles.length === 0 &&
				<span>
					<em>No articles found</em>
				</span>
			}
			{ articles !== null && articles.length > 0 &&
				<ul className="dmg-tile-article-selector__article-list striped">
					{ articles.map( articleDisplay ) }
				</ul>
			}
			<nav className="dmg-tile-article-selector__nav">
				<ul>
					<li
						className={ navPreviousClassname }
						onClick={ () => setPage( page - 1 ) }
					>
						&laquo; Previous
					</li>
					<li
						className={ navNextClassname }
						onClick={ () => setPage( page + 1 ) }
					>
						Next &raquo;
					</li>
				</ul>
			</nav>
		</div>
	);
};

export default ArticleSelector;
